
local function select_edit_pos()

  local cur_line = renoise.song().selected_line_index
  local cur_track = renoise.song().selected_track_index
  local cur_col = renoise.song().selected_note_column_index 
  local cur_col_fx = renoise.song().selected_effect_column_index 
  
  return  { start_line = cur_line, end_line = cur_line, start_track = cur_track, 
            end_track = cur_track, start_column = cur_col, end_column = cur_col,
            fx_column = cur_col_fx} 
end

local function get_pattern (pattern_ix)
  local pattern_ix = renoise.song().selected_pattern_index
  local patterns = renoise.song().patterns
  return patterns[pattern_ix] , pattern_ix
end

local function cur_pattern ()
  local pattern_ix = renoise.song().selected_pattern_index
  return get_pattern (pattern_ix)
end

local function copy_sel (sel) 

  local selection = {} 
   
  local pattern = cur_pattern ()   
    
  if sel ~= nil then
    for i = sel.start_track , sel.end_track do
      for j = sel.start_line , sel.end_line do
         for k = sel.start_column, sel.end_column do           
             local note = pattern:track(i):line(j):note_column(k).note_value             
             local inst = pattern:track(i):line(j):note_column(k).instrument_value
             local vol = pattern:track(i):line(j):note_column(k).volume_value
             local pan = pattern:track(i):line(j):note_column(k).panning_value            
             
             selection[#selection+1] = { track = i , line = j , note_column = k , note_value = note, 
                                         instrument_value = inst,  volume_value = vol, panning_value = pan}
         end
      end
    end
  end
  
 return selection 
end
 
local function restore_sel (selection, n)

  if not(next(selection) == nil) then  
   
    local pattern = cur_pattern ()
  
    for i = 1 , #selection do
      local tr = selection[i].track
      local ln  = selection[i].line + n
      if (ln <= 1) then
        ln = 1
      end
      
      local col = selection[i].note_column
      local note = selection[i].note_value
      local inst = selection[i].instrument_value
      local vol = selection[i].volume_value
      local pan = selection[i].panning_value        
  
      pattern:track(tr):line(ln):note_column(col).note_value = note
      pattern:track(tr):line(ln):note_column(col).instrument_value = inst
      pattern:track(tr):line(ln):note_column(col).volume_value = vol
      pattern:track(tr):line(ln):note_column(col).panning_value = pan
    end
  end  
end


local function nudge(addon)

   local td = {}
   local se = renoise.song().selection_in_pattern
   local se2 = {}
      
   if (se == nil) then
     se = select_edit_pos()
     se2 = select_edit_pos()
   else
     se2 = renoise.song().selection_in_pattern
   end
   
  local sel  = copy_sel (se)
   
  if (addon < 0) then

    if (se.start_line >= 2) then
      se2.start_line = se.start_line + addon
      se2.end_line = se2.start_line  
    
      td  = copy_sel (se2)
      restore_sel (sel,addon)
      rprint(td)
      for i=1,#td do
        td[i].line = se.end_line
      end
      restore_sel (td,0)
    else
      se2.start_line = se.start_line
      se2.end_line = se.start_line  
      
      td  = copy_sel (se2)
      rprint(td)
      restore_sel (sel,addon)
      
      for i=1,#td do
        td[i].line = se.end_line
      end
      restore_sel (td,0)
    end

  else 
    local pat_len = renoise.song().selected_pattern.number_of_lines
    
    if (se.end_line == pat_len) then 
    
      se2.start_line = se.end_line
      se2.end_line = se2.start_line
      
      td  = copy_sel (se2)
      rprint(td)
      restore_sel (sel,addon)
      
      for i=1,#td do
        td[i].line = se.start_line
      end
      restore_sel (td,0)      
    
    else
      se2.start_line = se.end_line + addon
      se2.end_line = se2.start_line
    
      td  = copy_sel (se2)
      restore_sel (sel,addon)
      rprint(td)  
      
      for i=1,#td do
        td[i].line = se.start_line
      end
      restore_sel (td,0)
    end
  end
end 


local function nudge_up()
 nudge (-1)
end

local function nudge_dw()
 nudge (1)
end
  

---------- Menu -------------------------------------------
  
renoise.tool():add_menu_entry {
  name = "Pattern Editor:Nudge Up",
  invoke = nudge_up
}

renoise.tool():add_menu_entry {
  name = "Pattern Editor:Nudge Down",
  invoke = nudge_dw
}

------- keys --------------------------
renoise.tool():add_keybinding {
  name = "Global:Tools:Nudge Up",
  invoke = nudge_up
}  
           
renoise.tool():add_keybinding {
  name = "Global:Tools:Nudge Down",
  invoke = nudge_dw
}       
